@@ -42,8 +42,10 @@
 #include <X11/extensions/dpmsstr.h>
 #endif
 
+#include "gpm-conf.h"
 #include "gpm-debug.h"
 #include "gpm-dpms.h"
+#include "gpm-hal.h"
 
 static void     gpm_dpms_class_init (GpmDpmsClass *klass);
 static void     gpm_dpms_init       (GpmDpms      *dpms);
@@ -53,16 +55,18 @@ static void     gpm_dpms_finalize   (GObject      *object);
 
 struct GpmDpmsPrivate
 {
-	gboolean       enabled;
-	gboolean       active;
+	gboolean		 enabled;
+	gboolean		 active;
 
-	guint	       standby_timeout;
-	guint	       suspend_timeout;
-	guint	       off_timeout;
+	guint			 standby_timeout;
+	guint			 suspend_timeout;
+	guint			 off_timeout;
 
-	GpmDpmsMode    mode;
+	GpmConf			*conf;
+	GpmDpmsMode		 mode;
+	GpmHal			*hal;
 
-	guint	       timer_id;
+	guint			 timer_id;
 };
 
 enum {
@@ -91,6 +95,16 @@ gpm_dpms_error_quark (void)
 	return quark;
 }
 
+gboolean
+gpm_dpms_has_hw (void)
+{
+#ifdef HAVE_DPMS_EXTENSION
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
 /* the following function is derived from
    xscreensaver Copyright (C) Jamie Zawinski
 */
@@ -566,9 +580,9 @@ gpm_dpms_mode_to_string (GpmDpmsMode mode)
 }
 
 gboolean
-gpm_dpms_set_mode (GpmDpms    *dpms,
-		   GpmDpmsMode mode,
-		   GError    **error)
+gpm_dpms_set_mode_enum (GpmDpms    *dpms,
+		        GpmDpmsMode mode,
+		        GError    **error)
 {
 	gboolean ret;
 
@@ -589,9 +603,9 @@ gpm_dpms_set_mode (GpmDpms    *dpms,
 }
 
 gboolean
-gpm_dpms_get_mode (GpmDpms     *dpms,
-		   GpmDpmsMode *mode,
-		   GError     **error)
+gpm_dpms_get_mode_enum (GpmDpms     *dpms,
+		        GpmDpmsMode *mode,
+		        GError     **error)
 {
 	gboolean ret;
 
@@ -778,7 +792,6 @@ poll_dpms_mode (GpmDpms *dpms)
 	return TRUE;
 }
 
-
 static void
 remove_poll_timer (GpmDpms *dpms)
 {
@@ -823,9 +836,11 @@ gpm_dpms_finalize (GObject *object)
 GpmDpms *
 gpm_dpms_new (void)
 {
-	GpmDpms *dpms;
-
-	dpms = g_object_new (GPM_TYPE_DPMS, NULL);
-
+	static GpmDpms *dpms = NULL;
+	if (dpms != NULL) {
+		g_object_ref (dpms);
+	} else {
+		dpms = g_object_new (GPM_TYPE_DPMS, NULL);
+	}
 	return GPM_DPMS (dpms);
 }
