@@ -44,7 +44,8 @@ struct GpmHalPrivate
 	DBusGConnection		*connection;
 	GHashTable		*watch_device_property_modified;
 	GHashTable		*watch_device_condition;
-	GpmProxy		*gproxy;
+	GpmProxy		*gproxy_manager;
+	GpmProxy		*gproxy_power;
 };
 
 /* Signals emitted from GpmHal are:
@@ -71,14 +72,13 @@ enum {
 };
 
 static guint	     signals [LAST_SIGNAL] = { 0, };
-static gpointer      gpm_hal_object = NULL;
 
 G_DEFINE_TYPE (GpmHal, gpm_hal, G_TYPE_OBJECT)
 
 /**
  * gpm_hal_is_running:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * Return value: TRUE if haldaemon is running
  *
  * Finds out if hal is running
@@ -99,7 +99,7 @@ gpm_hal_is_running (GpmHal *hal)
 /**
  * gpm_hal_device_rescan:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @udi: The HAL UDI
  * Return value: Success, true if we rescan'd
  *
@@ -140,7 +140,7 @@ gpm_hal_device_rescan (GpmHal *hal, const gchar *udi)
 /**
  * gpm_hal_device_rescan_capability:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @capability: The HAL capability, e.g. laptop_panel
  * Return value: Success, true if we found and devices
  *
@@ -170,7 +170,7 @@ gpm_hal_device_rescan_capability (GpmHal *hal, const gchar *capability)
 /**
  * gpm_hal_device_get_bool:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @udi: The UDI of the device
  * @key: The key to query
  * @value: return value, passed by ref
@@ -217,7 +217,7 @@ gpm_hal_device_get_bool (GpmHal      *hal,
 /**
  * gpm_hal_device_get_string:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @udi: The UDI of the device
  * @key: The key to query
  * @value: return value, passed by ref
@@ -266,7 +266,7 @@ gpm_hal_device_get_string (GpmHal      *hal,
 /**
  * gpm_hal_device_get_int:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @udi: The UDI of the device
  * @key: The key to query
  * @value: return value, passed by ref
@@ -333,7 +333,7 @@ gpm_hal_device_get_uint (GpmHal      *hal,
 /**
  * gpm_hal_device_find_capability:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @capability: The capability, e.g. "battery"
  * @value: return value, passed by ref
  * Return value: TRUE for success, FALSE for failure
@@ -377,7 +377,7 @@ gpm_hal_device_find_capability (GpmHal      *hal,
 /**
  * gpm_hal_device_has_capability:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @capability: The capability, e.g. "battery"
  * @value: return value, passed by ref
  * Return value: TRUE for success, FALSE for failure
@@ -423,7 +423,7 @@ gpm_hal_device_has_capability (GpmHal      *hal,
 /**
  * gpm_hal_free_capability:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @value: The list of strings to free
  *
  * Frees value result of gpm_hal_device_find_capability
@@ -445,7 +445,7 @@ gpm_hal_free_capability (GpmHal *hal, gchar **value)
 /**
  * gpm_hal_num_devices_of_capability:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @capability: The capability, e.g. "battery"
  * Return value: Number of devices of that capability
  *
@@ -475,7 +475,7 @@ gpm_hal_num_devices_of_capability (GpmHal *hal, const gchar *capability)
 /**
  * gpm_hal_num_devices_of_capability_with_value:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  * @capability: The capability, e.g. "battery"
  * @key: The key to match, e.g. "button.type"
  * @value: The key match, e.g. "power"
@@ -800,7 +800,7 @@ gpm_hal_device_remove_propery_modified (GpmHal      *hal,
 
 /**
  * gpm_hal_class_init:
- * @klass: This hal class instance
+ * @klass: This class instance
  **/
 static void
 gpm_hal_class_init (GpmHalClass *klass)
@@ -999,7 +999,7 @@ gpm_hal_proxy_connect_more (GpmHal *hal)
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 
-	proxy = gpm_proxy_get_proxy (hal->priv->gproxy);
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_manager);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
@@ -1050,7 +1050,7 @@ gpm_hal_proxy_disconnect_more (GpmHal *hal)
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 
-	proxy = gpm_proxy_get_proxy (hal->priv->gproxy);
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_manager);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
@@ -1086,7 +1086,7 @@ proxy_status_cb (DBusGProxy *proxy,
 /**
  * gpm_hal_init:
  *
- * @hal: This hal class instance
+ * @hal: This class instance
  **/
 static void
 gpm_hal_init (GpmHal *hal)
@@ -1102,14 +1102,29 @@ gpm_hal_init (GpmHal *hal)
 		g_error_free (error);
 	}
 
-	hal->priv->gproxy = gpm_proxy_new ();
-	proxy = gpm_proxy_assign (hal->priv->gproxy,
+	/* get the manager connection */
+	hal->priv->gproxy_manager = gpm_proxy_new ();
+	proxy = gpm_proxy_assign (hal->priv->gproxy_manager,
 				  GPM_PROXY_SYSTEM,
 				  HAL_DBUS_SERVICE,
 				  HAL_DBUS_PATH_MANAGER,
 				  HAL_DBUS_INTERFACE_MANAGER);
+	if (proxy == NULL) {
+		gpm_critical_error ("Either HAL or DBUS are not working!");
+	}
 
-	g_signal_connect (hal->priv->gproxy, "proxy-status",
+	/* get the power connection */
+	hal->priv->gproxy_power = gpm_proxy_new ();
+	proxy = gpm_proxy_assign (hal->priv->gproxy_power,
+				  GPM_PROXY_SYSTEM,
+				  HAL_DBUS_SERVICE,
+				  HAL_ROOT_COMPUTER,
+				  HAL_DBUS_INTERFACE_POWER);
+	if (proxy == NULL) {
+		gpm_critical_error ("HAL does not support power management!");
+	}
+
+	g_signal_connect (hal->priv->gproxy_manager, "proxy-status",
 			  G_CALLBACK (proxy_status_cb), hal);
 
 	hal->priv->watch_device_property_modified = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1149,9 +1164,451 @@ remove_device_condition_in_hash (const gchar *udi,
 	gpm_hal_device_remove_condition (hal, udi);
 }
 
+/**
+ * gpm_hal_is_laptop:
+ *
+ * @hal: This class instance
+ * Return value: TRUE is computer is identified as a laptop
+ *
+ * Returns true if system.formfactor is "laptop"
+ **/
+gboolean
+gpm_hal_is_laptop (GpmHal *hal)
+{
+	gboolean ret = TRUE;
+	gchar *formfactor = NULL;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	/* always present */
+	gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "system.formfactor", &formfactor);
+	if (formfactor == NULL) {
+		gpm_debug ("system.formfactor not set!");
+		/* no need to free */
+		return FALSE;
+	}
+	if (strcmp (formfactor, "laptop") != 0) {
+		gpm_debug ("This machine is not identified as a laptop."
+			   "system.formfactor is %s.", formfactor);
+		ret = FALSE;
+	}
+	g_free (formfactor);
+	return ret;
+}
+
+/**
+ * gpm_hal_has_power_management:
+ *
+ * @hal: This class instance
+ * Return value: TRUE if haldaemon has power management capability
+ *
+ * Finds out if power management functions are running (only ACPI, PMU, APM)
+ **/
+gboolean
+gpm_hal_has_power_management (GpmHal *hal)
+{
+	gchar *ptype = NULL;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "power_management.type", &ptype);
+	/* this key only has to exist to be pm okay */
+	if (ptype) {
+		gpm_debug ("Power management type : %s", ptype);
+		g_free (ptype);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+/**
+ * gpm_hal_can_suspend:
+ *
+ * @hal: This class instance
+ * Return value: TRUE if kernel suspend support is compiled in
+ *
+ * Finds out if HAL indicates that we can suspend
+ **/
+gboolean
+gpm_hal_can_suspend (GpmHal *hal)
+{
+	gboolean exists;
+	gboolean can_suspend;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	/* TODO: Change to can_suspend when rely on newer HAL */
+	exists = gpm_hal_device_get_bool (hal, HAL_ROOT_COMPUTER,
+					  "power_management.can_suspend_to_ram",
+					  &can_suspend);
+	if (exists == FALSE) {
+		gpm_warning ("gpm_hal_can_suspend: Key can_suspend_to_ram missing");
+		return FALSE;
+	}
+	return can_suspend;
+}
+
+/**
+ * gpm_hal_can_hibernate:
+ *
+ * @hal: This class instance
+ * Return value: TRUE if kernel hibernation support is compiled in
+ *
+ * Finds out if HAL indicates that we can hibernate
+ **/
+gboolean
+gpm_hal_can_hibernate (GpmHal *hal)
+{
+	gboolean exists;
+	gboolean can_hibernate;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	/* TODO: Change to can_hibernate when rely on newer HAL */
+	exists = gpm_hal_device_get_bool (hal, HAL_ROOT_COMPUTER,
+					  "power_management.can_suspend_to_disk",
+					  &can_hibernate);
+	if (exists == FALSE) {
+		gpm_warning ("gpm_hal_can_hibernate: Key can_suspend_to_disk missing");
+		return FALSE;
+	}
+	return can_hibernate;
+}
+
+/* we have to ignore dbus timeouts */
+static gboolean
+gpm_hal_filter_error (GError **error)
+{
+	/* short cut for speed, no error */
+	if (*error == NULL) {
+		return FALSE;
+	}
+
+	/* DBUS might time out, which is okay. We can remove this code
+	   when the dbus glib bindings are fixed. See #332888 */
+	if (g_error_matches (*error, DBUS_GERROR, DBUS_GERROR_NO_REPLY)) {
+		gpm_syslog ("DBUS timed out, but recovering");
+		g_error_free (*error);
+		*error = NULL;
+		return TRUE;
+	}
+	gpm_warning ("Method failed\n(%s)",
+		     (*error)->message);
+	gpm_syslog ("%s code='%i' quark='%s'", (*error)->message,
+		    (*error)->code, g_quark_to_string ((*error)->domain));
+	return FALSE;
+}
+
+/**
+ * gpm_hal_suspend:
+ *
+ * @hal: This class instance
+ * @wakeup: Seconds to wakeup, currently unsupported
+ * Return value: Success, true if we suspended OK
+ *
+ * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Suspend ()
+ **/
+gboolean
+gpm_hal_suspend (GpmHal *hal, guint wakeup)
+{
+	guint retval = 0;
+	GError *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_power);
+	if (proxy == NULL) {
+		gpm_warning ("not connected");
+		return FALSE;
+	}
+
+	ret = dbus_g_proxy_call (proxy, "Suspend", &error,
+				 G_TYPE_INT, wakeup,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, &retval,
+				 G_TYPE_INVALID);
+	/* we might have to ignore the error */
+	if (gpm_hal_filter_error (&error)) {
+		return TRUE;
+	}
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE || retval != 0) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("Suspend failed!");
+		return FALSE;
+	}
+	return TRUE;
+}
+
+/**
+ * hal_pm_method_void:
+ *
+ * @hal: This class instance
+ * @method: The method name, e.g. "Hibernate"
+ * Return value: Success, true if we did OK
+ *
+ * Do a method on org.freedesktop.Hal.Device.SystemPowerManagement.*
+ * with no arguments.
+ **/
+static gboolean
+hal_pm_method_void (GpmHal *hal, const gchar* method)
+{
+	guint retval = 0;
+	GError *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	g_return_val_if_fail (method != NULL, FALSE);
+
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_power);
+	if (proxy == NULL) {
+		gpm_warning ("not connected");
+		return FALSE;
+	}	
+
+	ret = dbus_g_proxy_call (proxy, method, &error,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, &retval,
+				 G_TYPE_INVALID);
+	/* we might have to ignore the error */
+	if (gpm_hal_filter_error (&error)) {
+		return TRUE;
+	}
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE || retval != 0) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("%s failed!", method);
+		return FALSE;
+	}
+	return TRUE;
+}
+
+/**
+ * gpm_hal_hibernate:
+ *
+ * @hal: This class instance
+ * Return value: Success, true if we hibernated OK
+ *
+ * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Hibernate ()
+ **/
+gboolean
+gpm_hal_hibernate (GpmHal *hal)
+{
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	return hal_pm_method_void (hal, "Hibernate");
+}
+
+/**
+ * gpm_hal_shutdown:
+ *
+ * Return value: Success, true if we shutdown OK
+ *
+ * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Shutdown ()
+ **/
+gboolean
+gpm_hal_shutdown (GpmHal *hal)
+{
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	return hal_pm_method_void (hal, "Shutdown");
+}
+
+/**
+ * gpm_hal_reboot:
+ *
+ * @hal: This class instance
+ * Return value: Success, true if we shutdown OK
+ *
+ * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Reboot ()
+ **/
+gboolean
+gpm_hal_reboot (GpmHal *hal)
+{
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	return hal_pm_method_void (hal, "Reboot");
+}
+
+/**
+ * gpm_hal_enable_power_save:
+ *
+ * @hal: This class instance
+ * @enable: True to enable low power mode
+ * Return value: Success, true if we set the mode
+ *
+ * Uses org.freedesktop.Hal.Device.SystemPowerManagement.SetPowerSave ()
+ **/
+gboolean
+gpm_hal_enable_power_save (GpmHal *hal, gboolean enable)
+{
+	gint retval = 0;
+	GError *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	g_return_val_if_fail (hal != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_power);
+	if (proxy == NULL) {
+		gpm_warning ("not connected");
+		return FALSE;
+	}
+
+	/* abort if we are not a "qualified" laptop */
+	if (gpm_hal_is_laptop (hal) == FALSE) {
+		gpm_debug ("We are not a laptop, so not even trying");
+		return FALSE;
+	}
+
+	gpm_debug ("Doing SetPowerSave (%i)", enable);
+	ret = dbus_g_proxy_call (proxy, "SetPowerSave", &error,
+				 G_TYPE_BOOLEAN, enable,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, &retval,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE || retval != 0) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("SetPowerSave failed!");
+		return FALSE;
+	}
+	return TRUE;
+}
+
+/**
+ * gpm_hal_has_suspend_error:
+ *
+ * @hal: This class instance
+ * @enable: Return true if there was a suspend error
+ * Return value: Success
+ *
+ * TODO: should call a method on HAL and also return the ouput of the file
+ **/
+gboolean
+gpm_hal_has_suspend_error (GpmHal *hal, gboolean *state)
+{
+	g_return_val_if_fail (hal != NULL, FALSE);
+	g_return_val_if_fail (state != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	*state = g_file_test ("/var/lib/hal/system-power-suspend-output", G_FILE_TEST_EXISTS);
+	return TRUE;
+}
+
+/**
+ * gpm_hal_has_hibernate_error:
+ *
+ * @hal: This class instance
+ * @enable: Return true if there was a hibernate error
+ * Return value: Success
+ *
+ * TODO: should call a method on HAL and also return the ouput of the file
+ **/
+gboolean
+gpm_hal_has_hibernate_error (GpmHal *hal, gboolean *state)
+{
+	g_return_val_if_fail (hal != NULL, FALSE);
+	g_return_val_if_fail (state != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	*state = g_file_test ("/var/lib/hal/system-power-hibernate-output", G_FILE_TEST_EXISTS);
+	return TRUE;
+}
+
+/**
+ * gpm_hal_clear_suspend_error:
+ *
+ * @hal: This class instance
+ * Return value: Success
+ *
+ * Tells HAL to try and clear the suspend error as we appear to be okay
+ **/
+gboolean
+gpm_hal_clear_suspend_error (GpmHal *hal)
+{
+#if HAVE_HAL_NEW
+	GError *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	g_return_val_if_fail (hal != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_power);
+	if (proxy == NULL) {
+		gpm_warning ("not connected");
+		return FALSE;
+	}
+
+	gpm_debug ("Doing SuspendClearError");
+	ret = dbus_g_proxy_call (proxy, "SuspendClearError", &error,
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("SuspendClearError failed!");
+		return FALSE;
+	}
+#endif
+	return TRUE;
+}
+
+/**
+ * gpm_hal_clear_hibernate_error:
+ *
+ * @hal: This class instance
+ * Return value: Success
+ *
+ * Tells HAL to try and clear the hibernate error as we appear to be okay
+ **/
+gboolean
+gpm_hal_clear_hibernate_error (GpmHal *hal)
+{
+#if HAVE_HAL_NEW
+	GError *error = NULL;
+	gboolean ret;
+	DBusGProxy *proxy;
+
+	g_return_val_if_fail (hal != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+
+	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_power);
+	if (proxy == NULL) {
+		gpm_warning ("not connected");
+		return FALSE;
+	}
+
+	gpm_debug ("Doing HibernateClearError");
+	ret = dbus_g_proxy_call (proxy, "HibernateClearError", &error,
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("HibernateClearError failed!");
+		return FALSE;
+	}
+#endif
+	return TRUE;
+}
+
 /**
  * gpm_hal_finalize:
- * @object: This hal class instance
+ * @object: This class instance
  **/
 static void
 gpm_hal_finalize (GObject *object)
@@ -1170,7 +1627,8 @@ gpm_hal_finalize (GObject *object)
 	g_hash_table_destroy (hal->priv->watch_device_property_modified);
 	g_hash_table_destroy (hal->priv->watch_device_condition);
 
-	g_object_unref (hal->priv->gproxy);
+	g_object_unref (hal->priv->gproxy_manager);
+	g_object_unref (hal->priv->gproxy_power);
 
 	G_OBJECT_CLASS (gpm_hal_parent_class)->finalize (object);
 }
@@ -1182,12 +1640,11 @@ gpm_hal_finalize (GObject *object)
 GpmHal *
 gpm_hal_new (void)
 {
-	if (gpm_hal_object) {
-		g_object_ref (gpm_hal_object);
+	static GpmHal *hal = NULL;
+	if (hal != NULL) {
+		g_object_ref (hal);
 	} else {
-		gpm_hal_object = g_object_new (GPM_TYPE_HAL, NULL);
-		g_object_add_weak_pointer (gpm_hal_object,
-					   (gpointer *) &gpm_hal_object);
+		hal = g_object_new (GPM_TYPE_HAL, NULL);
 	}
-	return GPM_HAL (gpm_hal_object);
+	return GPM_HAL (hal);
 }
