@@ -27,18 +27,20 @@
 
 #include <string.h>
 #include <time.h>
-#include <gconf/gconf-client.h>
 #include <dbus/dbus-gtype-specialized.h>
 
-#include "gpm-info.h"
-#include "gpm-info-data.h"
+#include "gpm-ac-adapter.h"
+#include "gpm-button.h"
+#include "gpm-conf.h"
 #include "gpm-common.h"
 #include "gpm-debug.h"
+#include "gpm-dpms.h"
 #include "gpm-hal.h"
-#include "gpm-hal-power.h"
+#include "gpm-info.h"
+#include "gpm-info-data.h"
 #include "gpm-power.h"
-#include "gpm-gconf.h"
 #include "gpm-stock-icons.h"
+#include "gpm-idle.h"
 
 static void     gpm_info_class_init (GpmInfoClass *klass);
 static void     gpm_info_init       (GpmInfo      *info);
@@ -55,9 +57,12 @@ static void     gpm_info_finalize   (GObject      *object);
 
 struct GpmInfoPrivate
 {
-	GpmPower		*power;
+	GpmAcAdapter		*ac_adapter;
+	GpmButton		*button;
+	GpmDpms			*dpms;
 	GpmHal			*hal;
-	GpmHalPower		*hal_power;
+	GpmIdle			*idle;
+	GpmPower		*power;
 
 	GpmInfoData		*events;
 	GpmInfoData		*rate_data;
@@ -344,12 +349,20 @@ gpm_info_log_do_poll (gpointer data)
 		gpm_info_data_add (info->priv->percentage_data,
 				   value_x,
 				   battery_status.percentage_charge, colour);
-		gpm_info_data_add (info->priv->rate_data,
-				   value_x,
-				   battery_status.charge_rate_raw, colour);
-		gpm_info_data_add (info->priv->time_data,
-				   value_x,
-				   battery_status.remaining_time, colour);
+
+		/* sanity check to less than 100W */
+		if (battery_status.charge_rate_raw < 100000) {
+			gpm_info_data_add (info->priv->rate_data,
+					   value_x,
+					   battery_status.charge_rate_raw, colour);
+		}
+
+		/* sanity check to less than 10 hours */
+		if (battery_status.charge_rate_raw < 10*60*60) {
+			gpm_info_data_add (info->priv->time_data,
+					   value_x,
+					   battery_status.remaining_time, colour);
+		}
 		gpm_info_data_add (info->priv->voltage,
 				   value_x,
 				   battery_status.voltage, colour);
@@ -357,6 +370,105 @@ gpm_info_log_do_poll (gpointer data)
 	return TRUE;
 }
 
+/**
+ * power_on_ac_changed_cb:
+ * @power: The power class instance
+ * @on_ac: if we are on AC power
+ * @manager: This class instance
+ *
+ * Does the actions when the ac power source is inserted/removed.
+ **/
+static void
+ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
+		       GpmAcAdapterState state,
+		       GpmInfo          *info)
+{
+	if (state == GPM_AC_ADAPTER_PRESENT) {
+		gpm_info_event_log (info, GPM_EVENT_ON_AC,
+				    _("AC adapter inserted"));
+	} else {
+		gpm_info_event_log (info, GPM_EVENT_ON_BATTERY,
+				    _("AC adapter removed"));
+	}
+}
+
+/**
+ * button_pressed_cb:
+ * @power: The power class instance
+ * @type: The button type, e.g. "power"
+ * @state: The state, where TRUE is depressed or closed
+ * @brightness: This class instance
+ **/
+static void
+button_pressed_cb (GpmButton   *power,
+		   const gchar *type,
+		   GpmInfo     *info)
+{
+	gpm_debug ("Button press event type=%s", type);
+
+	if (strcmp (type, GPM_BUTTON_LID_CLOSED) == 0) {
+		gpm_info_event_log (info,
+				    GPM_EVENT_LID_CLOSED,
+				    _("The laptop lid has been closed"));
+		gpm_debug ("lid button CLOSED");
+
+	} else if (strcmp (type, GPM_BUTTON_LID_OPEN) == 0) {
+		gpm_info_event_log (info,
+				    GPM_EVENT_LID_OPENED,
+				    _("The laptop lid has been re-opened"));
+		gpm_debug ("lid button OPENED");
+	}
+}
+
+/**
+ * idle_changed_cb:
+ * @idle: The idle class instance
+ * @mode: The idle mode, e.g. GPM_IDLE_MODE_SESSION
+ * @manager: This class instance
+ *
+ * This callback is called when gnome-screensaver detects that the idle state
+ * has changed. GPM_IDLE_MODE_SESSION is when the session has become inactive,
+ * and GPM_IDLE_MODE_SYSTEM is where the session has become inactive, AND the
+ * session timeout has elapsed for the idle action.
+ **/
+static void
+idle_changed_cb (GpmIdle     *idle,
+		 GpmIdleMode  mode,
+		 GpmInfo     *info)
+{
+	if (mode == GPM_IDLE_MODE_NORMAL) {
+		gpm_info_event_log (info, GPM_EVENT_SESSION_ACTIVE, _("idle mode ended"));
+
+	} else if (mode == GPM_IDLE_MODE_SESSION) {
+		gpm_info_event_log (info, GPM_EVENT_SESSION_IDLE, _("idle mode started"));
+	}
+}
+
+/**
+ * dpms_mode_changed_cb:
+ * @mode: The DPMS mode, e.g. GPM_DPMS_MODE_OFF
+ * @info: This class instance
+ *
+ * Log when the DPMS mode is changed.
+ **/
+static void
+dpms_mode_changed_cb (GpmDpms    *dpms,
+		      GpmDpmsMode mode,
+		      GpmInfo *info)
+{
+	gpm_debug ("DPMS mode changed: %d", mode);
+
+	if (mode == GPM_DPMS_MODE_ON) {
+		gpm_info_event_log (info, GPM_EVENT_DPMS_ON, _("dpms on"));
+	} else if (mode == GPM_DPMS_MODE_STANDBY) {
+		gpm_info_event_log (info, GPM_EVENT_DPMS_STANDBY, _("dpms standby"));
+	} else if (mode == GPM_DPMS_MODE_SUSPEND) {
+		gpm_info_event_log (info, GPM_EVENT_DPMS_SUSPEND, _("dpms suspend"));
+	} else if (mode == GPM_DPMS_MODE_OFF) {
+		gpm_info_event_log (info, GPM_EVENT_DPMS_OFF, _("dpms off"));
+	}
+}
+
 /**
  * gpm_info_class_init:
  * @klass: This info class instance
@@ -382,17 +494,36 @@ gpm_info_init (GpmInfo *info)
 	info->priv->start_time = time (NULL);
 
 	info->priv->hal = gpm_hal_new ();
-	info->priv->hal_power = gpm_hal_power_new ();
 
 	/* set up the timer callback so we can log data */
 	g_timeout_add (GPM_INFO_DATA_POLL * 1000, gpm_info_log_do_poll, info);
 
 	/* find out if we should log and display the extra graphs */
-	info->priv->is_laptop = gpm_hal_power_is_laptop (info->priv->hal_power);
+	info->priv->is_laptop = gpm_hal_is_laptop (info->priv->hal);
 
 	/* singleton, so okay */
 	info->priv->power = gpm_power_new ();
 
+	/* we use ac_adapter so we can log the event */
+	info->priv->ac_adapter = gpm_ac_adapter_new ();
+	g_signal_connect (info->priv->ac_adapter, "ac-adapter-changed",
+			  G_CALLBACK (ac_adapter_changed_cb), info);
+
+	/* watch for lid open/close */
+	info->priv->button = gpm_button_new ();
+	g_signal_connect (info->priv->button, "button-pressed",
+			  G_CALLBACK (button_pressed_cb), info);
+
+	/* watch for idle mode changes */
+	info->priv->idle = gpm_idle_new ();
+	g_signal_connect (info->priv->idle, "idle-changed",
+			  G_CALLBACK (idle_changed_cb), info);
+
+	/* watch for dpms mode changes */
+	info->priv->dpms = gpm_dpms_new ();
+	g_signal_connect (info->priv->dpms, "mode-changed",
+			  G_CALLBACK (dpms_mode_changed_cb), info);
+
 	/* set to a blank list */
 	info->priv->events = gpm_info_data_new ();
 	if (info->priv->is_laptop) {
@@ -409,9 +540,10 @@ gpm_info_init (GpmInfo *info)
 
 	if (info->priv->is_laptop) {
 		/* get the maximum x-axis size from gconf */
-		GConfClient *client = gconf_client_get_default ();
-		int max_time = gconf_client_get_int (client, GPM_PREF_GRAPH_DATA_MAX_TIME, NULL);
-		g_object_unref (client);
+		GpmConf *conf = gpm_conf_new ();
+		guint max_time;
+		gpm_conf_get_uint (conf, GPM_CONF_GRAPH_DATA_MAX_TIME, &max_time);
+		g_object_unref (conf);
 
 		gpm_info_data_set_max_time (info->priv->events, max_time);
 		gpm_info_data_set_max_time (info->priv->percentage_data, max_time);
@@ -447,10 +579,18 @@ gpm_info_finalize (GObject *object)
 	if (info->priv->voltage) {
 		g_object_unref (info->priv->voltage);
 	}
+	if (info->priv->ac_adapter != NULL) {
+		g_object_unref (info->priv->ac_adapter);
+	}
+	if (info->priv->idle != NULL) {
+		g_object_unref (info->priv->idle);
+	}
+	if (info->priv->dpms != NULL) {
+		g_object_unref (info->priv->dpms);
+	}
 	g_object_unref (info->priv->events);
 	g_object_unref (info->priv->power);
 	g_object_unref (info->priv->hal);
-	g_object_unref (info->priv->hal_power);
 
 	G_OBJECT_CLASS (gpm_info_parent_class)->finalize (object);
 }
