@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -51,8 +52,6 @@ typedef struct
 	 GObjectClass	parent_class;
 	 void		(* on_ac_changed)		(GpmManager	*manager,
 							 gboolean	 on_ac);
-	 void		(* dpms_mode_changed)		(GpmManager	*manager,
-							 const char	*mode);
 } GpmManagerClass;
 
 typedef enum
@@ -67,7 +66,10 @@ GType		 gpm_manager_get_type		  	(void);
 GpmManager	*gpm_manager_new			(void);
 
 gboolean	 gpm_manager_get_on_ac			(GpmManager	*manager,
-							 gboolean	*on_ac,
+							 gboolean	*retval,
+							 GError		**error);
+gboolean	 gpm_manager_get_low_power_mode		(GpmManager	*manager,
+							 gboolean	*retval,
 							 GError		**error);
 gboolean	 gpm_manager_allowed_shutdown		(GpmManager	*manager,
 							 gboolean	*allowed,
@@ -81,12 +83,6 @@ gboolean	 gpm_manager_allowed_suspend		(GpmManager	*manager,
 gboolean	 gpm_manager_allowed_hibernate		(GpmManager	*manager,
 							 gboolean	*allowed,
 							 GError		**error);
-gboolean	 gpm_manager_get_dpms_mode		(GpmManager	*manager,
-							 const char	**mode,
-							 GError		**error);
-gboolean	 gpm_manager_set_dpms_mode		(GpmManager	*manager,
-							 const char	*mode,
-							 GError		**error);
 gboolean	 gpm_manager_suspend_dbus_method	(GpmManager	*manager,
 							 GError		**error);
 gboolean	 gpm_manager_hibernate_dbus_method	(GpmManager	*manager,
@@ -96,12 +92,12 @@ gboolean	 gpm_manager_shutdown_dbus_method	(GpmManager	*manager,
 gboolean	 gpm_manager_reboot_dbus_method		(GpmManager	*manager,
 						 	 GError		**error);
 void		 gpm_manager_inhibit			(GpmManager	*manager,
-						   	 const char	*application,
-							 const char	*reason,
+						   	 const gchar	*application,
+							 const gchar	*reason,
 							 DBusGMethodInvocation *context,
 							 GError		**error);
 void		 gpm_manager_uninhibit			(GpmManager	*manager,
-						 	 int		 cookie,
+						 	 guint32	 cookie,
 				   			 DBusGMethodInvocation *context,
 				   			 GError		**error);
 
