@@ -89,11 +89,10 @@ G_DEFINE_TYPE (GpmPrefs, gpm_prefs, G_TYPE_OBJECT)
 #define CPUFREQ_ONDEMAND_TEXT		_("Based on processor load")
 #define CPUFREQ_CONSERVATIVE_TEXT	_("Automatic power saving")
 #define CPUFREQ_POWERSAVE_TEXT		_("Maximum power saving")
-#define CPUFREQ_USERSPACE_TEXT		_("Custom")
-#define CPUFREQ_PERFORMANCE_TEXT	_("Maximum speed")
-#define CPUFREQ_NOTHING_TEXT 		_("Do nothing")
+#define CPUFREQ_PERFORMANCE_TEXT	_("Always maximum speed")
 
-#define GPM_SLEEP_MAX_TIME		60
+/* If sleep time in a slider is set to 61 it is considered as never */
+const int NEVER_TIME_ON_SLIDER = 61;
 
 /**
  * gpm_prefs_class_init:
@@ -238,7 +237,11 @@ gpm_prefs_format_time_cb (GtkScale *scale,
 			  GpmPrefs *prefs)
 {
 	gchar *str;
-	str = gpm_get_timestring (value * 60);
+	if ((gint) value == NEVER_TIME_ON_SLIDER) {
+		str = g_strdup (_("Never"));
+	} else {
+		str = gpm_get_timestring (value * 60);
+	}
 	return str;
 }
 
@@ -257,14 +260,19 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
 
 	value = (int) gtk_range_get_value (range);
 
-	/* We take away the g-s idle time as the slider represents
-	 * global time but we only do our timeout from when g-s
-	 * declares the session idle */
-	gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	value -= gs_idle_time;
+	if (value == NEVER_TIME_ON_SLIDER) {
+		/* power manager interprets 0 as Never */
+		value = 0;
+	} else {
+		/* We take away the g-s idle time as the slider represents
+		 * global time but we only do our timeout from when g-s
+		 * declares the session idle */
+		gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
+		value -= gs_idle_time;
 
-	/* policy is in seconds, slider is in minutes */
-	value *= 60;
+		/* policy is in seconds, slider is in minutes */
+		value *= 60;
+	}
 
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "conf_key");
 	gpm_debug ("Changing %s to %i", gpm_pref_key, value);
@@ -296,10 +304,14 @@ gpm_prefs_setup_sleep_slider (GpmPrefs    *prefs,
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
-	/* policy is in seconds, slider is in minutes */
-	value /= 60;
-	gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	value += gs_idle_time;
+	if (value == 0) {
+		value = NEVER_TIME_ON_SLIDER;
+	} else {
+		/* policy is in seconds, slider is in minutes */
+		value /= 60;
+		gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
+		value += gs_idle_time;
+	}
 
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
@@ -368,26 +380,6 @@ gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
 	return widget;
 }
 
-/**
- * gpm_prefs_enable_widget:
- **/
-static gboolean
-gpm_prefs_enable_widget (GpmPrefs    *prefs,
-		       const gchar *widgetname,
-		       gboolean     show)
-{
-	GtkWidget *widget;
-
-	widget = glade_xml_get_widget (prefs->priv->glade_xml, widgetname);
-	if (widget == NULL) {
-		gpm_warning ("widget '%s' not found", widgetname);
-		return FALSE;
-	}
-
-	gtk_widget_set_sensitive (widget, show);
-	return TRUE;
-}
-
 /**
  * gpm_prefs_action_combo_changed_cb:
  * @widget: The GtkWidget object
@@ -400,7 +392,6 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	gchar *value;
 	const gchar *action;
 	gchar *gpm_pref_key;
-	const gchar *widget_name;
 
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 
@@ -424,13 +415,6 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	gpm_debug ("Changing %s to %s", gpm_pref_key, action);
 	gpm_conf_set_string (prefs->priv->conf, gpm_pref_key, action);
-
-	widget_name = gtk_widget_get_name (widget);
-	if (widget_name && strcmp (widget_name, "combobox_sleep_ac_type") == 0) {
-		gpm_prefs_enable_widget (prefs, "hbox_sleep_ac_custom", action != ACTION_NOTHING);
-	} else if (widget_name && strcmp (widget_name, "combobox_sleep_battery_type") == 0) {
-		gpm_prefs_enable_widget (prefs, "hbox_sleep_battery_custom", action != ACTION_NOTHING);
-	}
 }
 
 /**
@@ -522,27 +506,11 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
 {
 	gboolean checked;
 	gchar *gpm_pref_key;
-	GtkWidget *twidget;
 	const gchar *widget_name;
 
 	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
 	widget_name = gtk_widget_get_name (widget);
-	if (widget_name && strcmp (widget_name, "checkbutton_display_state_change") == 0) {
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"label_display_ac_brightness");
-		gtk_widget_set_sensitive (twidget, checked);
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"hscale_display_ac_brightness");
-		gtk_widget_set_sensitive (twidget, checked);
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"label_display_battery_brightness");
-		gtk_widget_set_sensitive (twidget, checked);
-		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
-						"hscale_display_battery_brightness");
-		gtk_widget_set_sensitive (twidget, checked);
-	}
-
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	gpm_debug ("Changing %s to %i", gpm_pref_key, checked);
 	gpm_conf_set_bool (prefs->priv->conf, gpm_pref_key, checked);
@@ -554,7 +522,7 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
  * @widget_name: The GtkWidget name
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
-static GtkWidget*
+static GtkWidget *
 gpm_prefs_setup_checkbox (GpmPrefs    *prefs,
 			  const gchar *widget_name,
 			  const gchar *gpm_pref_key)
@@ -624,12 +592,12 @@ set_idle_hscale_stops (GpmPrefs    *prefs,
 {
 	GtkWidget *widget;
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
-	if (gs_idle_time + 1 > GPM_SLEEP_MAX_TIME) {
+	if (gs_idle_time + 1 > NEVER_TIME_ON_SLIDER) {
 		gpm_warning ("gnome-screensaver timeout is really big. "
 			     "Not sure what to do");
 		return;
 	}
-	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, GPM_SLEEP_MAX_TIME);
+	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, NEVER_TIME_ON_SLIDER);
 }
 
 /**
@@ -675,59 +643,6 @@ conf_key_changed_cb (GpmConf     *conf,
 	}
 }
 
-/**
- * gpm_prefs_processor_slider_changed_cb:
- * @range: The GtkRange object
- * @gpm_pref_key: The GConf key for this preference setting.
- **/
-static void
-gpm_prefs_processor_slider_changed_cb (GtkRange *range,
-				       GpmPrefs *prefs)
-{
-	gint value;
-	gchar *gpm_pref_key;
-
-	value = (gint) gtk_range_get_value (range);
-	gpm_pref_key = (gchar *) g_object_get_data (G_OBJECT (range), "conf_key");
-	gpm_debug ("Changing %s to %i", gpm_pref_key, value);
-	gpm_conf_set_int (prefs->priv->conf, gpm_pref_key, value);
-}
-
-/**
- * gpm_prefs_setup_processor_slider:
- * @prefs: This prefs class instance
- * @widget_name: The GtkWidget name
- * @gpm_pref_key: The GConf key for this preference setting.
- **/
-static GtkWidget *
-gpm_prefs_setup_processor_slider (GpmPrefs   *prefs,
-				  const gchar *widget_name,
-				  const gchar *gpm_pref_key)
-{
-	GtkWidget *widget;
-	gint value;
-	gboolean is_writable;
-
-	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
-	g_signal_connect (G_OBJECT (widget), "format-value",
-			  G_CALLBACK (gpm_prefs_format_percentage_cb), prefs);
-
-	gpm_conf_get_int (prefs->priv->conf, gpm_pref_key, &value);
-	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
-
-	gtk_widget_set_sensitive (widget, is_writable);
-
-	gtk_range_set_value (GTK_RANGE (widget), value);
-
-	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
-
-	g_signal_connect (G_OBJECT (widget), "value-changed",
-			  G_CALLBACK (gpm_prefs_processor_slider_changed_cb),
-			  prefs);
-
-	return widget;
-}
-
 /**
  * gpm_prefs_processor_combo_changed_cb:
  * @widget: The GtkWidget object
@@ -740,7 +655,6 @@ gpm_prefs_processor_combo_changed_cb (GtkWidget *widget,
 	gchar *value;
 	const gchar *policy;
 	gchar *gpm_pref_key;
-	gboolean show_custom = FALSE;
 
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 	if (value == NULL) {
@@ -753,24 +667,12 @@ gpm_prefs_processor_combo_changed_cb (GtkWidget *widget,
 		policy = CODE_CPUFREQ_CONSERVATIVE;
 	} else if (strcmp (value, CPUFREQ_POWERSAVE_TEXT) == 0) {
 		policy = CODE_CPUFREQ_POWERSAVE;
-	} else if (strcmp (value, CPUFREQ_USERSPACE_TEXT) == 0) {
-		policy = CODE_CPUFREQ_USERSPACE;
-		show_custom = TRUE;
 	} else if (strcmp (value, CPUFREQ_PERFORMANCE_TEXT) == 0) {
 		policy = CODE_CPUFREQ_PERFORMANCE;
-	} else if (strcmp (value, CPUFREQ_NOTHING_TEXT) == 0) {
-		policy = CODE_CPUFREQ_NOTHING;
 	} else {
 		g_assert (FALSE);
 	}
 
-	/* show other options */
-	if (strcmp (gtk_widget_get_name (widget), "combobox_processor_ac_profile") == 0) {
-		gpm_prefs_enable_widget (prefs, "hbox_processor_ac_custom", show_custom);
-	} else {
-		gpm_prefs_enable_widget (prefs, "hbox_processor_battery_custom", show_custom);
-	}
-
 	g_free (value);
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	gpm_debug ("Changing %s to %s", gpm_pref_key, policy);
@@ -842,15 +744,6 @@ gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_USERSPACE) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-					   CPUFREQ_USERSPACE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_USERSPACE) {
-			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
-			has_option = TRUE;
-		}
-		n_added++;
-	}
 	if (cpufreq_types & GPM_CPUFREQ_PERFORMANCE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_PERFORMANCE_TEXT);
@@ -861,235 +754,12 @@ gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 		n_added++;
 	}
 
-	/* always add nothing as a valid option */
-	gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-				   CPUFREQ_NOTHING_TEXT);
 	if (has_option == FALSE || cpufreq_type == GPM_CPUFREQ_NOTHING) {
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 	}
 	g_free (value);
 }
 
-/** setup the sleep page */
-static void
-prefs_setup_sleep (GpmPrefs *prefs)
-{
-	GtkWidget *widget;
-	gint delay;
-
-	const gchar *sleep_type_actions[] = {ACTION_NOTHING,
-					     ACTION_SUSPEND,
-					     ACTION_HIBERNATE,
-					     NULL};
-
-	/* Sleep Type Combo Box */
-	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_ac_type",
-				      GPM_CONF_AC_SLEEP_TYPE,
-				      sleep_type_actions);
-	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_battery_type",
-				      GPM_CONF_BATTERY_SLEEP_TYPE,
-				      sleep_type_actions);
-	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_ups_type",
-				      GPM_CONF_BATTERY_SLEEP_TYPE, /* fixme */
-				      sleep_type_actions);
-
-	/* Sleep time until we sleep */
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ac_inactive",
-				      GPM_CONF_AC_SLEEP_COMPUTER);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_battery_inactive",
-				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ups_inactive",
-				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
-
-	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_sleep_ac_inactive", delay);
-	set_idle_hscale_stops (prefs, "hscale_sleep_battery_inactive", delay);
-	set_idle_hscale_stops (prefs, "hscale_sleep_ups_inactive", delay);
-
-	if (prefs->priv->has_batteries == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_sleep_battery");
-		gtk_widget_hide_all (widget);
-	}
-	if (prefs->priv->has_ups == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_sleep_ups");
-		gtk_widget_hide_all (widget);
-	}
-}
-
-/** setup the display page */
-static void
-prefs_setup_display (GpmPrefs *prefs)
-{
-	GtkWidget *widget;
-	gint delay;
-
-	/* Sleep time for display to blank */
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_display_ac_sleep",
-				      GPM_CONF_AC_SLEEP_DISPLAY);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_display_battery_sleep",
-				      GPM_CONF_BATTERY_SLEEP_DISPLAY);
-	/* Display brightness */
-	gpm_prefs_setup_brightness_slider (prefs, "hscale_display_ac_brightness",
-					   GPM_CONF_AC_BRIGHTNESS);
-	gpm_prefs_setup_brightness_slider (prefs, "hscale_display_battery_brightness",
-					   GPM_CONF_BATTERY_BRIGHTNESS);
-
-	/* set up the general checkboxes */
-	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_dim",
-				  GPM_CONF_DISPLAY_IDLE_DIM);
-	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_state_change",
-				  GPM_CONF_DISPLAY_STATE_CHANGE);
-	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_ambient",
-				  GPM_CONF_DISPLAY_STATE_CHANGE);
-	/* for now, hide */
-	widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_ambient");
-	gtk_widget_hide_all (widget);
-
-	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_display_battery_sleep", delay);
-	set_idle_hscale_stops (prefs, "hscale_display_ac_sleep", delay);
-
-	if (prefs->priv->has_lcd == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_display_ac_brightness");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_display_battery_brightness");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_dim");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_state_change");
-		gtk_widget_hide_all (widget);
-	}
-
-	/* the box will be empty */
-	if (prefs->priv->has_lcd == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_display_general");
-		gtk_widget_hide_all (widget);
-	}
-
-	if (prefs->priv->has_batteries == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_display_battery");
-		gtk_widget_hide_all (widget);
-	}
-}
-
-/** setup the processor page */
-static void
-prefs_setup_processor (GpmPrefs *prefs)
-{
-	GtkWidget    *widget;
-	GtkWidget    *notebook;
-	gint	      page;
-
-	/* remove if we have no options to set */
-	if (prefs->priv->cpufreq == NULL) {
-		notebook = glade_xml_get_widget (prefs->priv->glade_xml, "notebook_preferences");
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_processor");
-		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
-		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
-		return;
-	}
-
-	gpm_prefs_setup_processor_slider (prefs, "hscale_processor_battery_custom",
-					  GPM_CONF_BATTERY_CPUFREQ_VALUE);
-	gpm_prefs_setup_processor_slider (prefs, "hscale_processor_ac_custom",
-					  GPM_CONF_AC_CPUFREQ_VALUE);
-
-	gpm_prefs_setup_processor_combo (prefs, "combobox_processor_ac_profile",
-					 GPM_CONF_AC_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
-	gpm_prefs_setup_processor_combo (prefs, "combobox_processor_battery_profile",
-					 GPM_CONF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
-}
-
-/** setup the actions page */
-static void
-prefs_setup_actions (GpmPrefs *prefs)
-{
-	GtkWidget    *widget;
-	const gchar  *button_lid_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_BLANK,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 NULL};
-	const gchar  *suspend_button_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 NULL};
-	const gchar  *battery_critical_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
-	const gchar  *battery_ups_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
-	const gchar  *power_button_actions[] =
-				{ACTION_INTERACTIVE,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
-	/* Power button action */
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_general_power",
-				      GPM_CONF_BUTTON_POWER,
-				      power_button_actions);
-
-	/* Suspend button action */
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_general_suspend",
-				      GPM_CONF_BUTTON_SUSPEND,
-				      suspend_button_actions);
-
-	/* Lid close actions */
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ac_lid",
-				      GPM_CONF_AC_BUTTON_LID,
-				      button_lid_actions);
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_battery_lid",
-				      GPM_CONF_BATTERY_BUTTON_LID,
-				      button_lid_actions);
-
-	/* set up the LowPowerMode checkbox */
-	gpm_prefs_setup_checkbox (prefs, "checkbutton_actions_battery_low_power",
-	  			  GPM_CONF_BATTERY_LOWPOWER);
-
-	/* battery critical actions */
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_battery_critical",
-				      GPM_CONF_BATTERY_CRITICAL,
-				      battery_critical_actions);
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_critical",
-				      GPM_CONF_UPS_CRITICAL,
-				      battery_ups_actions);
-	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_low",
-				      GPM_CONF_UPS_LOW,
-				      battery_ups_actions);
-
-	if (prefs->priv->has_button_suspend == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_actions_general_suspend");
-		gtk_widget_hide_all (widget);
-	}
-
-	if (prefs->priv->has_button_lid == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_actions_battery_lid");
-		gtk_widget_hide_all (widget);
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_actions_ac_lid");
-		gtk_widget_hide_all (widget);
-		/* remove whole box as there is nothing in it */
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_ac");
-		gtk_widget_hide_all (widget);
-	}
-	if (prefs->priv->has_ups == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_ups");
-		gtk_widget_hide_all (widget);
-	}
-	if (prefs->priv->has_batteries == FALSE) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_battery");
-		gtk_widget_hide_all (widget);
-	}
-}
-
 /** setup the notification page */
 static void
 prefs_setup_notification (GpmPrefs *prefs)
@@ -1164,6 +834,10 @@ prefs_setup_notification (GpmPrefs *prefs)
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_notification_sound",
 	  			  GPM_CONF_ENABLE_BEEPING);
 
+	if (prefs->priv->has_batteries == TRUE) {
+		/* there's no use case for displaying this option */
+		gtk_widget_hide_all (radiobutton_icon_never);
+	}
 	if (prefs->priv->has_batteries == FALSE) {
 		/* Hide battery radio options if we have no batteries */
 		gtk_widget_hide_all (radiobutton_icon_charge);
@@ -1175,6 +849,177 @@ prefs_setup_notification (GpmPrefs *prefs)
 	}
 }
 
+static void
+prefs_setup_ac (GpmPrefs *prefs)
+{
+	GtkWidget *widget;
+	gint delay;
+	const gchar  *button_lid_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_BLANK,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 NULL};
+	gpm_prefs_setup_action_combo (prefs, "combobox_ac_lid",
+				      GPM_CONF_AC_BUTTON_LID,
+				      button_lid_actions);
+	gpm_prefs_setup_processor_combo (prefs, "combobox_ac_cpu",
+					 GPM_CONF_AC_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_computer",
+				      GPM_CONF_AC_SLEEP_COMPUTER);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_display",
+				      GPM_CONF_AC_SLEEP_DISPLAY);
+	gpm_prefs_setup_brightness_slider (prefs, "hscale_ac_brightness",
+					   GPM_CONF_AC_BRIGHTNESS);
+
+	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
+	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
+
+	if (prefs->priv->has_button_lid == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_lid");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->cpufreq == NULL) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_cpu");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->has_lcd == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_brightness");
+		gtk_widget_hide_all (widget);
+	}
+}
+
+static void
+prefs_setup_battery (GpmPrefs *prefs)
+{
+	GtkWidget *widget;
+	GtkWidget *notebook;
+	gint delay;
+	gint page;
+	
+	const gchar  *button_lid_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_BLANK,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 NULL};
+	const gchar  *battery_critical_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 ACTION_SHUTDOWN,
+				 NULL};
+
+	if (prefs->priv->has_batteries == FALSE) {
+		notebook = glade_xml_get_widget (prefs->priv->glade_xml, "notebook_preferences");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_battery");
+		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
+		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
+		return;
+	}
+
+	gpm_prefs_setup_action_combo (prefs, "combobox_battery_lid",
+				      GPM_CONF_BATTERY_BUTTON_LID,
+				      button_lid_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_battery_critical",
+				      GPM_CONF_BATTERY_CRITICAL,
+				      battery_critical_actions);
+	gpm_prefs_setup_processor_combo (prefs, "combobox_battery_cpu",
+					 GPM_CONF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_computer",
+				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_display",
+				      GPM_CONF_BATTERY_SLEEP_DISPLAY);
+	gpm_prefs_setup_brightness_slider (prefs, "hscale_battery_brightness",
+					   GPM_CONF_BATTERY_BRIGHTNESS);
+
+	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
+	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
+
+	if (prefs->priv->has_button_lid == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_lid");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->cpufreq == NULL) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_cpu");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->has_lcd == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_brightness");
+		gtk_widget_hide_all (widget);
+	}
+}
+
+static void
+prefs_setup_ups (GpmPrefs *prefs)
+{
+	GtkWidget *widget;
+	GtkWidget *notebook;
+	gint delay;
+	gint page;
+
+	const gchar  *ups_low_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_HIBERNATE,
+				 ACTION_SHUTDOWN,
+				 NULL};
+
+	if (prefs->priv->has_ups == FALSE) {
+		notebook = glade_xml_get_widget (prefs->priv->glade_xml, "notebook_preferences");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_ups");
+		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
+		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
+		return;
+	}
+
+	gpm_prefs_setup_action_combo (prefs, "combobox_ups_low",
+				      GPM_CONF_UPS_LOW,
+				      ups_low_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_ups_critical",
+				      GPM_CONF_UPS_CRITICAL,
+				      ups_low_actions);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_ups_computer",
+				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	set_idle_hscale_stops (prefs, "hscale_ups_computer", delay);
+}
+
+static void
+prefs_setup_general (GpmPrefs *prefs)
+{
+	GtkWidget *widget;
+	const gchar  *power_button_actions[] =
+				{ACTION_INTERACTIVE,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 ACTION_SHUTDOWN,
+				 NULL};
+	const gchar  *suspend_button_actions[] =
+				{ACTION_NOTHING,
+				 ACTION_SUSPEND,
+				 ACTION_HIBERNATE,
+				 NULL};
+
+	gpm_prefs_setup_action_combo (prefs, "combobox_general_power",
+				      GPM_CONF_BUTTON_POWER,
+				      power_button_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_general_suspend",
+				      GPM_CONF_BUTTON_SUSPEND,
+				      suspend_button_actions);
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_general_ambient",
+				  GPM_CONF_DISPLAY_STATE_CHANGE);
+	/* for now, hide */
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_general_ambient");
+	gtk_widget_hide_all (widget);
+
+	if (prefs->priv->has_button_suspend == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_general_suspend");
+		gtk_widget_hide_all (widget);
+	}
+}
+
 /**
  * gpm_prefs_init:
  * @prefs: This prefs class instance
@@ -1211,7 +1056,6 @@ gpm_prefs_init (GpmPrefs *prefs)
 	prefs->priv->has_button_suspend = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
 							"button.type",
 							"suspend") > 0;
-
 	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
 	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
 
@@ -1247,19 +1091,10 @@ gpm_prefs_init (GpmPrefs *prefs)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_prefs_help_cb), prefs);
 
-	/* the 'Sleep' tab */
-	prefs_setup_sleep (prefs);
-
-	/* the 'Display' tab */
-	prefs_setup_display (prefs);
-
-	/* the 'Processing Speed' tab */
-	prefs_setup_processor (prefs);
-
-	/* the 'Actions' tab */
-	prefs_setup_actions (prefs);
-
-	/* the 'Notification' tab */
+	prefs_setup_ac (prefs);
+	prefs_setup_battery (prefs);
+	prefs_setup_ups (prefs);
+	prefs_setup_general (prefs);
 	prefs_setup_notification (prefs);
 
 	gtk_widget_show (main_window);
