@@ -25,8 +25,8 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
-#include <gconf/gconf-client.h>
 
+#include "gpm-conf.h"
 #include "gpm-screensaver.h"
 #include "gpm-debug.h"
 #include "gpm-proxy.h"
@@ -48,13 +48,14 @@ static void     gpm_screensaver_finalize   (GObject		*object);
 struct GpmScreensaverPrivate
 {
 	GpmProxy		*gproxy;
-	GConfClient		*gconf_client;
-	int			 idle_delay;	/* the setting in g-s-p, cached */
+	GpmConf			*conf;
+	guint			 idle_delay;	/* the setting in g-s-p, cached */
 };
 
 enum {
 	GS_DELAY_CHANGED,
 	CONNECTION_CHANGED,
+	IDLE_CHANGED,
 	AUTH_REQUEST,
 	LAST_SIGNAL
 };
@@ -70,9 +71,8 @@ static void
 gpm_screensaver_auth_begin (DBusGProxy     *proxy,
 			    GpmScreensaver *screensaver)
 {
-	const gboolean value = TRUE;
-	gpm_debug ("emitting auth-request : (%i)", value);
-	g_signal_emit (screensaver, signals [AUTH_REQUEST], 0, value);
+	gpm_debug ("emitting auth-request : (%i)", TRUE);
+	g_signal_emit (screensaver, signals [AUTH_REQUEST], 0, TRUE);
 }
 
 /** Invoked when we get the AuthenticationRequestEnd from g-s when the user
@@ -82,14 +82,25 @@ static void
 gpm_screensaver_auth_end (DBusGProxy     *proxy,
 			  GpmScreensaver *screensaver)
 {
-	const gboolean value = FALSE;
-	gpm_debug ("emitting auth-request : (%i)", value);
-	g_signal_emit (screensaver, signals [AUTH_REQUEST], 0, value);
+	gpm_debug ("emitting auth-request : (%i)", FALSE);
+	g_signal_emit (screensaver, signals [AUTH_REQUEST], 0, FALSE);
+}
+
+/** Invoked when we get the AuthenticationRequestEnd from g-s when the user
+ *  has entered a valid password or re-authenticated.
+ */
+static void
+gpm_screensaver_idle_changed (DBusGProxy     *proxy,
+			      gboolean        is_idle,
+			      GpmScreensaver *screensaver)
+{
+	gpm_debug ("emitting idle-changed : (%i)", is_idle);
+	g_signal_emit (screensaver, signals [IDLE_CHANGED], 0, is_idle);
 }
 
 /**
  * gpm_screensaver_proxy_connect_more:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  **/
 static gboolean
 gpm_screensaver_proxy_connect_more (GpmScreensaver *screensaver)
@@ -107,26 +118,32 @@ gpm_screensaver_proxy_connect_more (GpmScreensaver *screensaver)
 	g_signal_emit (screensaver, signals [CONNECTION_CHANGED], 0, TRUE);
 
 	/* get AuthenticationRequestBegin */
-	dbus_g_proxy_add_signal (proxy,
-				 "AuthenticationRequestBegin", G_TYPE_INVALID);
+	dbus_g_proxy_add_signal (proxy, "AuthenticationRequestBegin", G_TYPE_INVALID);
 	dbus_g_proxy_connect_signal (proxy,
 				     "AuthenticationRequestBegin",
 				     G_CALLBACK (gpm_screensaver_auth_begin),
 				     screensaver, NULL);
 
 	/* get AuthenticationRequestEnd */
-	dbus_g_proxy_add_signal (proxy,
-				 "AuthenticationRequestEnd", G_TYPE_INVALID);
+	dbus_g_proxy_add_signal (proxy, "AuthenticationRequestEnd", G_TYPE_INVALID);
 	dbus_g_proxy_connect_signal (proxy,
 				     "AuthenticationRequestEnd",
 				     G_CALLBACK (gpm_screensaver_auth_end),
 				     screensaver, NULL);
+
+	/* get SessionIdleChanged */
+	dbus_g_proxy_add_signal (proxy, "SessionIdleChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (proxy,
+				     "SessionIdleChanged",
+				     G_CALLBACK (gpm_screensaver_idle_changed),
+				     screensaver, NULL);
+
 	return TRUE;
 }
 
 /**
  * gpm_screensaver_proxy_disconnect_more:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  **/
 static gboolean
 gpm_screensaver_proxy_disconnect_more (GpmScreensaver *screensaver)
@@ -144,18 +161,14 @@ gpm_screensaver_proxy_disconnect_more (GpmScreensaver *screensaver)
  * Turn a gconf key change into a signal
  **/
 static void
-gconf_key_changed_cb (GConfClient  *client,
-		      guint	    cnxn_id,
-		      GConfEntry   *entry,
-		      gpointer	    user_data)
+gconf_key_changed_cb (GpmConf        *conf,
+		      const gchar    *key,
+		      GpmScreensaver *screensaver)
 {
-	GpmScreensaver *screensaver = GPM_SCREENSAVER (user_data);
-
 	g_return_if_fail (GPM_IS_SCREENSAVER (screensaver));
-	g_return_if_fail (entry != NULL);
 
-	if (strcmp (entry->key, GS_PREF_IDLE_DELAY) == 0) {
-		screensaver->priv->idle_delay = gconf_client_get_int (client, entry->key, NULL);
+	if (strcmp (key, GS_PREF_IDLE_DELAY) == 0) {
+		gpm_conf_get_uint (screensaver->priv->conf, key, &screensaver->priv->idle_delay);
 		gpm_debug ("emitting gs-delay-changed : %i", screensaver->priv->idle_delay);
 		g_signal_emit (screensaver, signals [GS_DELAY_CHANGED], 0, screensaver->priv->idle_delay);
 	}
@@ -163,34 +176,36 @@ gconf_key_changed_cb (GConfClient  *client,
 
 /**
  * gpm_screensaver_lock_enabled:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * Return value: If gnome-screensaver is set to lock the screen on screensave
  **/
 gboolean
 gpm_screensaver_lock_enabled (GpmScreensaver *screensaver)
 {
+	gboolean enabled;
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-	return gconf_client_get_bool (screensaver->priv->gconf_client,
-				      GS_PREF_LOCK_ENABLED, NULL);
+
+	gpm_conf_get_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, &enabled);
+
+	return enabled;
 }
 
 /**
  * gpm_screensaver_lock_set:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * @lock: If gnome-screensaver should lock the screen on screensave
  **/
 gboolean
 gpm_screensaver_lock_set (GpmScreensaver *screensaver, gboolean lock)
 {
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-	gconf_client_set_bool (screensaver->priv->gconf_client,
-			       GS_PREF_LOCK_ENABLED, lock, NULL);
+	gpm_conf_set_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, lock);
 	return TRUE;
 }
 
 /**
  * gpm_screensaver_get_delay:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * Return value: The delay for the idle time set in gnome-screesaver-properties.
  **/
 gint
@@ -202,7 +217,7 @@ gpm_screensaver_get_delay (GpmScreensaver *screensaver)
 
 /**
  * gpm_screensaver_lock
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * Return value: Success value
  **/
 gboolean
@@ -243,7 +258,7 @@ gpm_screensaver_lock (GpmScreensaver *screensaver)
 
 /**
  * gpm_screensaver_add_throttle:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * @reason:      The reason for throttling
  * Return value: Success value, or zero for failure
  **/
@@ -272,7 +287,7 @@ gpm_screensaver_add_throttle (GpmScreensaver *screensaver,
 	}
 
 	ret = dbus_g_proxy_call (proxy, "Throttle", &error,
-				 G_TYPE_STRING, "Power Manager",
+				 G_TYPE_STRING, "Power screensaver",
 				 G_TYPE_STRING, reason,
 				 G_TYPE_INVALID,
 				 G_TYPE_UINT, &cookie,
@@ -327,7 +342,7 @@ gpm_screensaver_remove_throttle (GpmScreensaver *screensaver,
 
 /**
  * gpm_screensaver_check_running:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * Return value: TRUE if gnome-screensaver is running
  **/
 gboolean
@@ -360,7 +375,7 @@ gpm_screensaver_check_running (GpmScreensaver *screensaver)
 
 /**
  * gpm_screensaver_poke:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  *
  * Pokes GNOME Screensaver simulating hardware events. This displays the unlock
  * dialogue when we resume, so the user doesn't have to move the mouse or press
@@ -388,7 +403,7 @@ gpm_screensaver_poke (GpmScreensaver *screensaver)
 
 /**
  * gpm_screensaver_get_idle:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  * @time_secs: The returned idle time, passed by ref
  * Return value: Success value.
  **/
@@ -427,7 +442,7 @@ gpm_screensaver_get_idle (GpmScreensaver *screensaver, gint *time_secs)
 
 /**
  * gpm_screensaver_class_init:
- * @klass: This screensaver class instance
+ * @klass: This class instance
  **/
 static void
 gpm_screensaver_class_init (GpmScreensaverClass *klass)
@@ -465,13 +480,23 @@ gpm_screensaver_class_init (GpmScreensaverClass *klass)
 			      NULL,
 			      g_cclosure_marshal_VOID__BOOLEAN,
 			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+
+	signals [IDLE_CHANGED] =
+		g_signal_new ("idle-changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmScreensaverClass, idle_changed),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__BOOLEAN,
+			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 }
 
 /**
  * proxy_status_cb:
  * @proxy: The dbus raw proxy
  * @status: The status of the service, where TRUE is connected
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  **/
 static void
 proxy_status_cb (DBusGProxy     *proxy,
@@ -489,7 +514,7 @@ proxy_status_cb (DBusGProxy     *proxy,
 
 /**
  * gpm_screensaver_init:
- * @screensaver: This screensaver class instance
+ * @screensaver: This class instance
  **/
 static void
 gpm_screensaver_init (GpmScreensaver *screensaver)
@@ -511,26 +536,18 @@ gpm_screensaver_init (GpmScreensaver *screensaver)
 
 	gpm_screensaver_proxy_connect_more (screensaver);
 
-	screensaver->priv->gconf_client = gconf_client_get_default ();
-
+	screensaver->priv->conf = gpm_conf_new ();
+	g_signal_connect (screensaver->priv->conf, "value-changed",
+			  G_CALLBACK (gconf_key_changed_cb), screensaver);
 
 	/* get value of delay in g-s-p */
-	screensaver->priv->idle_delay = gconf_client_get_int (screensaver->priv->gconf_client,
-							      GS_PREF_IDLE_DELAY, NULL);
-
-	/* set up the monitoring for gnome-screensaver */
-	gconf_client_add_dir (screensaver->priv->gconf_client,
-			      GS_PREF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
-
-	gconf_client_notify_add (screensaver->priv->gconf_client,
-				 GS_PREF_DIR,
-				 gconf_key_changed_cb,
-				 screensaver, NULL, NULL);
+	gpm_conf_get_uint (screensaver->priv->conf, GS_PREF_IDLE_DELAY,
+			   &screensaver->priv->idle_delay);
 }
 
 /**
  * gpm_screensaver_finalize:
- * @object: This screensaver class instance
+ * @object: This class instance
  **/
 static void
 gpm_screensaver_finalize (GObject *object)
@@ -543,7 +560,7 @@ gpm_screensaver_finalize (GObject *object)
 	screensaver->priv = GPM_SCREENSAVER_GET_PRIVATE (screensaver);
 
 	gpm_screensaver_proxy_disconnect_more (screensaver);
-	g_object_unref (screensaver->priv->gconf_client);
+	g_object_unref (screensaver->priv->conf);
 	g_object_unref (screensaver->priv->gproxy);
 
 	G_OBJECT_CLASS (gpm_screensaver_parent_class)->finalize (object);
