@@ -27,30 +27,25 @@
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <dbus/dbus-glib.h>
-#include <gconf/gconf-client.h>
 #include <math.h>
 #include <string.h>
 
 #include "gpm-common.h"
 #include "gpm-prefs.h"
-#include "gpm-gconf.h"
+#include "gpm-conf.h"
 #include "gpm-statistics-core.h"
 #include "gpm-debug.h"
 #include "gpm-stock-icons.h"
-#include "gpm-graph-widget.h"
+#include "gpm-info.h"
 #include "gpm-info-data.h"
 #include "gpm-proxy.h"
+#include "gpm-powermanager.h"
 
 static void     gpm_statistics_class_init (GpmStatisticsClass *klass);
 static void     gpm_statistics_init       (GpmStatistics      *statistics);
 static void     gpm_statistics_finalize   (GObject	    *object);
 
 #define GPM_STATISTICS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_STATISTICS, GpmStatisticsPrivate))
-#define	GPM_DBUS_SERVICE		"org.gnome.PowerManager"
-#define	GPM_DBUS_PATH			"/org/gnome/PowerManager"
-#define	GPM_DBUS_PATH_STATS		"/org/gnome/PowerManager/Statistics"
-#define	GPM_DBUS_INTERFACE		"org.gnome.PowerManager"
-#define	GPM_DBUS_INTERFACE_STATS	"org.gnome.PowerManager.Statistics"
 
 #define ACTION_VOLTAGE		"voltage"
 #define ACTION_CHARGE		"charge"
@@ -66,8 +61,8 @@ static void     gpm_statistics_finalize   (GObject	    *object);
 struct GpmStatisticsPrivate
 {
 	GladeXML		*glade_xml;
-	GConfClient		*gconf_client;
 	GtkWidget		*graph_widget;
+	GpmConf			*conf;
 	GpmProxy		*gproxy;
 	GpmInfoData		*events;
 	GpmInfoData		*data;
@@ -169,28 +164,19 @@ gpm_statistics_delete_event_cb (GtkWidget	*widget,
 }
 
 /**
- * gconf_key_changed_cb:
+ * conf_key_changed_cb:
  *
  * We might have to do things when the gconf keys change; do them here.
  **/
 static void
-gconf_key_changed_cb (GConfClient *client,
-		      guint	   cnxn_id,
-		      GConfEntry  *entry,
-		      gpointer	   user_data)
+conf_key_changed_cb (GpmConf       *conf,
+		     const gchar   *key,
+		     GpmStatistics *statistics)
 {
-	GpmStatistics *statistics = GPM_STATISTICS (user_data);
 	gboolean  enabled;
 
-	gpm_debug ("Key changed %s", entry->key);
-
-	if (gconf_entry_get_value (entry) == NULL) {
-		return;
-	}
-
-	if (strcmp (entry->key, GPM_PREF_AC_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (statistics->priv->gconf_client,
-				  		 GPM_PREF_AC_LOWPOWER, NULL);
+	if (strcmp (key, GPM_CONF_AC_LOWPOWER) == 0) {
+		gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_AC_LOWPOWER, &enabled);
 		gpm_debug ("need to enable checkbox");
 	}
 }
@@ -310,8 +296,7 @@ gpm_statistics_checkbox_events_cb (GtkWidget     *widget,
 	gpm_debug ("Events enable %i", checked);
 
 	/* save to gconf so we open next time with the correct setting */
-	gconf_client_set_bool (statistics->priv->gconf_client,
-			       GPM_PREF_STAT_SHOW_EVENTS, checked, NULL);
+	gpm_conf_set_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_EVENTS, checked);
 
 	if (checked == FALSE) {
 		/* remove the dots from the graph */
@@ -341,8 +326,7 @@ gpm_statistics_checkbox_legend_cb (GtkWidget *widget,
 	gpm_debug ("Legend enable %i", checked);
 
 	/* save to gconf so we open next time with the correct setting */
-	gconf_client_set_bool (statistics->priv->gconf_client,
-			       GPM_PREF_STAT_SHOW_LEGEND, checked, NULL);
+	gpm_conf_set_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_LEGEND, checked);
 
 	gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), checked);
 }
@@ -419,7 +403,7 @@ gpm_statistics_free_list_strings (GList *list)
 }
 
 static gboolean
-gpm_statistics_get_data (GpmStatistics *statistics,
+gpm_statistics_get_data_dbus (GpmStatistics *statistics,
 			 const gchar *type)
 {
 	GError *error = NULL;
@@ -491,7 +475,7 @@ gpm_statistics_get_data (GpmStatistics *statistics,
 }
 
 static gboolean
-gpm_statistics_get_axis_type (GpmStatistics          *statistics,
+gpm_statistics_get_axis_type_dbus (GpmStatistics          *statistics,
 			      const gchar 	     *type,
 			      GpmGraphWidgetAxisType *x,
 			      GpmGraphWidgetAxisType *y)
@@ -543,7 +527,12 @@ static void
 gpm_statistics_refresh_data (GpmStatistics *statistics)
 {
 	GList *list = NULL;
-	gpm_statistics_get_data (statistics, statistics->priv->graph_type);
+
+	/* only get the data for a valid type */
+	if (statistics->priv->graph_type != NULL) {
+		gpm_statistics_get_data_dbus (statistics, statistics->priv->graph_type);
+	}
+
 	list = gpm_info_data_get_list (statistics->priv->data);
 
 	gpm_graph_widget_set_data (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), list);
@@ -576,7 +565,7 @@ gpm_statistics_type_combo_changed_cb (GtkWidget      *widget,
 	g_free (value);
 
 	/* find out what sort of grid axis we need */
-	gpm_statistics_get_axis_type (statistics, type, &axis_x, &axis_y);
+	gpm_statistics_get_axis_type_dbus (statistics, type, &axis_x, &axis_y);
 	gpm_graph_widget_set_axis_x (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), axis_x);
 	gpm_graph_widget_set_axis_y (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), axis_y);
 
@@ -584,8 +573,7 @@ gpm_statistics_type_combo_changed_cb (GtkWidget      *widget,
 	statistics->priv->graph_type = type;
 
 	/* save in gconf so we choose the correct graph type on next startup */
-	gconf_client_set_string (statistics->priv->gconf_client,
-				 GPM_PREF_STAT_GRAPH_TYPE, type, NULL);
+	gpm_conf_set_string (statistics->priv->conf, GPM_CONF_STAT_GRAPH_TYPE, type);
 
 	/* refresh data automatically */
 	gpm_statistics_refresh_data (statistics);
@@ -608,8 +596,7 @@ gpm_statistics_populate_graph_types (GpmStatistics *statistics,
 		return;
 	}
 
-	saved = gconf_client_get_string (statistics->priv->gconf_client,
-					 GPM_PREF_STAT_GRAPH_TYPE, NULL);
+	gpm_conf_get_string (statistics->priv->conf, GPM_CONF_STAT_GRAPH_TYPE, &saved);
 	/* gconf error, bahh */
 	if (saved == NULL) {
 		saved = g_strdup ("power");
@@ -683,7 +670,9 @@ gpm_statistics_init (GpmStatistics *statistics)
 
 	statistics->priv = GPM_STATISTICS_GET_PRIVATE (statistics);
 
-	statistics->priv->gconf_client = gconf_client_get_default ();
+	statistics->priv->conf = gpm_conf_new ();
+	g_signal_connect (statistics->priv->conf, "value-changed",
+			  G_CALLBACK (conf_key_changed_cb), statistics);
 
 	glade_set_custom_handler (gpm_graph_widget_custom_handler, statistics);
 
@@ -694,12 +683,11 @@ gpm_statistics_init (GpmStatistics *statistics)
 			  GPM_DBUS_PATH_STATS,
 			  GPM_DBUS_INTERFACE_STATS);
 
-	gconf_client_notify_add (statistics->priv->gconf_client,
-				 GPM_PREF_DIR,
-				 gconf_key_changed_cb,
-				 statistics,
-				 NULL,
-				 NULL);
+	/* would happen if not using g-p-m or using an old version of g-p-m */
+	if (gpm_proxy_is_connected (statistics->priv->gproxy) == FALSE) {
+		gpm_critical_error ("%s\n%s", _("Could not connect to GNOME Power Manager."),
+				    _("Perhaps the program is not running or you are using an old version?"));
+	}
 
 	statistics->priv->graph_type = NULL;
 	statistics->priv->events = gpm_info_data_new ();
@@ -731,6 +719,79 @@ gpm_statistics_init (GpmStatistics *statistics)
 	statistics->priv->graph_widget = widget;
 	gpm_graph_widget_set_axis_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
 
+	/* add the key items */
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("On AC"),
+				  GPM_EVENT_ON_AC,
+				  GPM_GRAPH_WIDGET_COLOUR_BLUE,
+				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("On battery"),
+				  GPM_EVENT_ON_BATTERY,
+				  GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE,
+				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Session idle"),
+				  GPM_EVENT_SESSION_IDLE,
+				  GPM_GRAPH_WIDGET_COLOUR_YELLOW,
+				  GPM_GRAPH_WIDGET_SHAPE_SQUARE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Session active"),
+				  GPM_EVENT_SESSION_ACTIVE,
+				  GPM_GRAPH_WIDGET_COLOUR_DARK_YELLOW,
+				  GPM_GRAPH_WIDGET_SHAPE_SQUARE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Suspend"),
+				  GPM_EVENT_SUSPEND,
+				  GPM_GRAPH_WIDGET_COLOUR_RED,
+				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Resume"),
+				  GPM_EVENT_RESUME,
+				  GPM_GRAPH_WIDGET_COLOUR_DARK_RED,
+				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Hibernate"),
+				  GPM_EVENT_HIBERNATE,
+				  GPM_GRAPH_WIDGET_COLOUR_MAGENTA,
+				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Lid closed"),
+				  GPM_EVENT_LID_CLOSED,
+				  GPM_GRAPH_WIDGET_COLOUR_GREEN,
+				  GPM_GRAPH_WIDGET_SHAPE_TRIANGLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Lid opened"),
+				  GPM_EVENT_LID_OPENED,
+				  GPM_GRAPH_WIDGET_COLOUR_DARK_GREEN,
+				  GPM_GRAPH_WIDGET_SHAPE_TRIANGLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("Notification"),
+				  GPM_EVENT_NOTIFICATION,
+				  GPM_GRAPH_WIDGET_COLOUR_GREY,
+				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("DPMS On"),
+				  GPM_EVENT_DPMS_ON,
+				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
+				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("DPMS Standby"),
+				  GPM_EVENT_DPMS_STANDBY,
+				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
+				  GPM_GRAPH_WIDGET_SHAPE_TRIANGLE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("DPMS Suspend"),
+				  GPM_EVENT_DPMS_SUSPEND,
+				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
+				  GPM_GRAPH_WIDGET_SHAPE_SQUARE);
+	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
+				  _("DPMS Off"),
+				  GPM_EVENT_DPMS_OFF,
+				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
+				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
+				       
+
 	/* FIXME: There's got to be a better way than this */
 	gtk_widget_hide (GTK_WIDGET (widget));
 	gtk_widget_show (GTK_WIDGET (widget));
@@ -744,16 +805,14 @@ gpm_statistics_init (GpmStatistics *statistics)
 	gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
 
 	widget = glade_xml_get_widget (statistics->priv->glade_xml, "checkbutton_events");
-	checked = gconf_client_get_bool (statistics->priv->gconf_client,
-					 GPM_PREF_STAT_SHOW_EVENTS, NULL);
+	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_EVENTS, &checked);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_statistics_checkbox_events_cb), statistics);
 	gpm_statistics_checkbox_events_cb (widget, statistics);
 
 	widget = glade_xml_get_widget (statistics->priv->glade_xml, "checkbutton_legend");
-	checked = gconf_client_get_bool (statistics->priv->gconf_client,
-					 GPM_PREF_STAT_SHOW_LEGEND, NULL);
+	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_LEGEND, &checked);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_statistics_checkbox_legend_cb), statistics);
@@ -778,7 +837,7 @@ gpm_statistics_finalize (GObject *object)
 	statistics = GPM_STATISTICS (object);
 	statistics->priv = GPM_STATISTICS_GET_PRIVATE (statistics);
 
-	g_object_unref (statistics->priv->gconf_client);
+	g_object_unref (statistics->priv->conf);
 	g_object_unref (statistics->priv->gproxy);
 	g_object_unref (statistics->priv->events);
 	g_object_unref (statistics->priv->data);
