@@ -24,7 +24,6 @@
 #define __GPM_TRAY_ICON_H
 
 #include <glib-object.h>
-#include "eggtrayicon.h"
 
 G_BEGIN_DECLS
 
@@ -39,43 +38,25 @@ typedef struct GpmTrayIconPrivate GpmTrayIconPrivate;
 
 typedef struct
 {
-	EggTrayIcon	    parent;
+	GObject		    parent;
 	GpmTrayIconPrivate *priv;
 } GpmTrayIcon;
 
 typedef struct
 {
-	EggTrayIconClass			 parent_class;
+	GObjectClass	parent_class;
 	void	(* suspend)			(GpmTrayIcon *tray_icon);
 	void	(* hibernate)			(GpmTrayIcon *tray_icon);
 } GpmTrayIconClass;
 
-typedef enum {
-	GPM_NOTIFY_URGENCY_CRITICAL = 1,
-	GPM_NOTIFY_URGENCY_NORMAL   = 2,
-	GPM_NOTIFY_URGENCY_LOW      = 3
-} GpmNotifyLevel;
-
 GType		 gpm_tray_icon_get_type			(void);
 GpmTrayIcon	*gpm_tray_icon_new			(void);
 
-void		 gpm_tray_icon_notify			(GpmTrayIcon	*icon,
-							 const char	*title,
-							 const char	*content,
-							 guint		 timeout,
-							 const char	*msgicon,
-							 GpmNotifyLevel	 urgency);
-void		 gpm_tray_icon_cancel_notify		(GpmTrayIcon	*icon);
 void		 gpm_tray_icon_set_tooltip		(GpmTrayIcon	*icon,
-							 const char	*tooltip);
+							 const gchar	*tooltip);
 void		 gpm_tray_icon_set_image_from_stock	(GpmTrayIcon	*icon,
-							 const char	*stock_id);
-void		 gpm_tray_icon_enable_suspend		(GpmTrayIcon	*icon,
-							 gboolean	 enabled);
-void		 gpm_tray_icon_enable_hibernate		(GpmTrayIcon	*icon,
-							 gboolean	 enabled);
-void		 gpm_tray_icon_show			(GpmTrayIcon	*icon,
-							 gboolean	 enabled);
+							 const gchar	*stock_id);
+void		 gpm_tray_icon_sync			(GpmTrayIcon	*icon);
 
 G_END_DECLS
 
